import psycopg2


class Database(object):
    def __init__(self):
        pass

    def get_conn(self, dbusername=None, dbpassword=None, dbhost='localhost', dbname='TheTraveler'):
        conn = psycopg2.connect(host=dbhost, dbname=dbname, user=dbusername, password=dbpassword)
        return conn


class oAuthDatabase(Database):
    def __init__(self, dbusername, dbpassword, dbhost, dbname):
        super(oAuthDatabase, self).__init__()
        self.oAuth_conn = self.get_conn(dbusername, dbpassword, dbhost, dbname)
        with self.oAuth_conn as conn:
            with conn.cursor() as c:
                c.execute("SET CLIENT_ENCODING TO 'UTF8';")

    def get_accounts(self, id):
        with self.oAuth_conn as conn:
            with conn.cursor('get_accounts') as c:
                c.execute("SELECT app_id, app_secret, username, password FROM oauth_data WHERE agent_of=%s", (id,))
                return c.fetchall()



#SOMETHING TO HANDLE REDDIT'S BULLSHIT

class InvalidAddition(Exception):
    def __init__(self, *args, **kwargs):
        super(InvalidAddition, self).__init__(*args, **kwargs)
        

class TooFrequent(Exception):
    def __init__(self, waitTime, *args, **kwargs):
        super(TooFrequent, self).__init__(*args, **kwargs)
        self.waitTime = waitTime.seconds//60 + 1




from .logger import LoggerConfig
from .database import Database
from .exceptions import InvalidAddition, TooFrequent
from .auth import oAuth

__all__ = ['LoggerConfig', 'Database', 'InvalidAddition', 'TooFrequent', 'oAuth']


"""Layer 7 Utilities constants."""

__app_name__ = "Layer-7-Utilities"
__description__ = "This is a set of utilities used by https://Layer7.Solutions for the software tools we create. It includes a logger with default configuration information we setup as well as an oAuth wrapper to be able to pull login information from a custom database."
__author__ = "Layer 7 Solutions (Mike Wohlrab)"
__version__ = "1.2.10"



import os
import json


class LoggerConfig(object):
    def __init__(
        self,
        __dsn__=None,
        __app_name__="_DefaultName",
        __version__="0.0.0",
        logspath="/opt/skynet/RedditBots/logs/",
    ):

        configfilepath = os.path.join(os.path.dirname(__file__), "logging_config.json")
        with open(configfilepath, "rt") as f:
            self.config = json.load(f)

        # Replaces the filename with one specific to each bot
        try:
            self.config["handlers"]["rotateFileHandler"][
                "filename"
            ] = f"{logspath}_DefaultName_Logs.log".replace(
                "_DefaultName", __app_name__.replace(" ", "_")
            )
        except Exception:
            raise AttributeError("Unable to set normal logging configuration location")

        try:
            self.config["handlers"]["rotateFileHandler_debug"][
                "filename"
            ] = f"{logspath}_DefaultName_Logs_Debug.log".replace(
                "_DefaultName", __app_name__
            )
        except Exception:
            raise AttributeError("Unable to set debug logging configuration location")

        # Replace the Sentry DSN ID with the app specific one
        try:
            self.config["handlers"]["SentryHandler"]["dsn"] = __dsn__
        except Exception:
            raise AttributeError("Unable to set Sentry DSN info")

        # Set the App Version / Release Version
        self.config["handlers"]["SentryHandler"]["release"] = __version__

    def get_config(self):
        return self.config



from setuptools import setup
from layer7_utilities import const

with open("ReadMe.md", "r") as fh:
    long_description = fh.read()

setup(
    name=const.__app_name__,
    version=const.__version__,
    description=const.__description__,
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://Layer7.Solutions",
    author=const.__author__,
    author_email="Mike@Layer7.Solutions",
    license="MIT",
    packages=[
        "layer7_utilities",
        "layer7_utilities.logger",
        "layer7_utilities.exceptions",
        "layer7_utilities.database",
        "layer7_utilities.auth",
    ],
    install_requires=[
        "psycopg2",
        "raven",
    ],
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
